﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions
{
    [Item("CF1", "The unconstrained CF1 test problem from the CEC 2009 special session and competition. // http://www.ntu.edu.sg/home/epnsugan/index_files/CEC-06/CEC06.htm")]
    [StorableClass]
    public class CF1 : MultiObjectiveTestFunction
    {

        public override double[,] Bounds(int objectives)
        {
            return new double[,] { { -4, 4 } };
        }

        public override bool[] Maximization(int objecitves)
        {
            return new bool[2];
        }

        public override int MaximumSolutionLength
        {
            get { return int.MaxValue; }
        }

        public override int MinimumSolutionLength
        {
            get { return 1; }
        }

        public override int MinimumObjectives
        {
            get { return 2; }
        }

        public override int MaximumObjectives
        {
            get { return 2; }
        }


        public override IEnumerable<double[]> OptimalParetoFront(int objectives)
        {
            return PFStore.get(this.ItemName);
        }
        public override double BestKnownHypervolume(int objectives)
        {
            return Hypervolume.Calculate(OptimalParetoFront(objectives), ReferencePoint(objectives), Maximization(objectives));
        }

        public override double[] ReferencePoint(int objectives)
        {
            return new double[] { 11, 11 };
        }

        [StorableConstructor]
        protected CF1(bool deserializing) : base(deserializing) { }
        protected CF1(CF1 original, Cloner cloner) : base(original, cloner) { }
        public override IDeepCloneable Clone(Cloner cloner)
        {
            return new CF1(this, cloner);
        }
        public CF1() : base() { }


        public override double[] Evaluate(RealVector r, int objectives)
        {
            return new double[]{r[0],r[0]};
            if (objectives != 2) throw new ArgumentException("The CF1 problem must always have 2 objectives");
            double f0 = 0.0, f1 = 0.0, c0 = 0.0;

            int j, count1, count2;
            double sum1, sum2, yj, N, a;
            N = 10.0; a = 1.0;
            sum1 = sum2 = 0.0;
            count1 = count2 = 0;
            for (j = 2; j <= r.Length; j++)
            {
                yj = r[j - 1] - Math.Pow(r[0], 0.5 * (1.0 + 3.0 * (j - 2.0) / (r.Length - 2.0)));
                if (j % 2 == 1)
                {
                    sum1 += yj * yj;
                    count1++;
                }
                else {
                    sum2 += yj * yj;
                    count2++;
                }
            }
            f0 = r[0] + 2.0 * sum1 / (double)count1;
            f1 = 1.0 - r[0] + 2.0 * sum2 / (double)count2;
            c0 = f1 + f0 - a * Math.Abs(Math.Sin(N * Math.PI * (f0 - f1 + 1.0))) - 1.0;

            double[] res = { f0, f1, c0 };
            return res;
        }
    }
}
